-- Replacement for open() that tries to handle Windows NT long file names.
-- Euphoria's open() already handles long filenames on Windows 95.
-- Returns failure if file can't be opened or there's any ambiguity.
-- Doesn't yet handle case where a directory name on the path is a long name.
-- Windows NT users can include this file in ed.ex, and change the calls to 
-- open() in ed.ex to calls to long_open(). It's not perfect, but it's
-- quite helpful.

include file.e

function slash(integer c)
    return c = '/' or c = '\\' 
end function

function file_type(sequence name)
-- extract file type
    for i = length(name) to 1 by -1 do
	if name[i] = '.' then
	    return name[i..length(name)] -- includes leading dot
	elsif slash(name[i]) then
	    return ""
	end if
    end for
    return ""
end function

function file_path(sequence name)
-- extract file path
    for i = length(name) to 1 by -1 do
	if slash(name[i]) or name[i] = ':' then
	    return name[1..i] 
	end if
    end for
    return ""
end function

function file_name(sequence name)
-- extract file name 
    for i = length(name) to 1 by -1 do
	if slash(name[i]) or name[i] = ':' then
	    name = name[i+1..length(name)]
	    exit
	end if
    end for
    for j = length(name) to 1 by -1 do
	if name[j] = '.' then
	    name = name[1..j-1]
	    exit
	end if
    end for
    return name
end function

function remove_dots(sequence name)
-- take out illegal dots from name
    integer d

    while 1 do
	-- remove extra dots in file name
	d = find('.', name)
	if d = 0 then
	    return name
	end if
	name = name[1..d-1] & name[d+1..length(name)]
    end while
end function

function upper(sequence s)
-- convert to upper case
    for i = 1 to length(s) do
	if s[i] >= 'a' and s[i] <= 'z' then
	    s[i] = s[i] - 'a'+ 'A' 
	end if
    end for
    return s
end function

function get_tilde(sequence fpath, sequence fname, sequence ftype)
-- return tilde value for short name
-- or "" if there are multiple short names that match
    object d
    sequence digits, value, dname, dtype
    integer t, count
    
    if length(fpath) = 0 then
	fpath = current_dir()
    elsif length(fpath) > 1 then
	if fpath[length(fpath)] != ':' then
	    fpath = fpath[1..length(fpath)-1] -- drop the slash
	end if
    end if
    
    d = dir(fpath)
    if atom(d) then
	return ""
    end if
    
    fname = upper(fname & '~')
    ftype = upper(ftype) 
    
    count = 0
    for i = 1 to length(d) do
	dname = d[i][D_NAME]
	if match(fname, dname) and not find('d', d[i][D_ATTRIBUTES]) then
	    t = find('~', dname)+1
	    -- skip digits part
	    digits = ""
	    while t <= length(dname) do
		if find(dname[t], "1234567890") then
		    digits = append(digits, dname[t])
		    t = t + 1
		else
		    exit
		end if
	    end while
	    dtype = dname[t..length(dname)] 
	    if compare(ftype, dtype) = 0 then
		count = count + 1
		value = '~' & digits
	    end if
	end if
    end for
    if count = 1 then
	return value
    else
	return ""
    end if
end function

global function long_open(sequence long_name, sequence mode)
-- open a file, possibly converting a long (NT) file name to
-- it's short DOS equivalent
    sequence fpath, fname, ftype, tilde
    
    fpath = file_path(long_name)
    fname = file_name(long_name)
    ftype = file_type(long_name)
    if length(fname) > 8 or length(ftype) > 4 or find('.', fname) then
	-- compute corresponding 8.3 DOS name
	fname = remove_dots(fname)
	if length(fname) > 6 then
	    fname = fname[1..6]
	end if
	if length(ftype) > 4 then
	    ftype = ftype[1..4]
	end if
	tilde = get_tilde(fpath, fname, ftype) 
	if length(tilde) = 0 then
	    return -1
	else
	    fname = fname & tilde
	    return open(fpath & fname & ftype, mode)
	end if
    else
	return open(long_name, mode)
    end if
end function

