-- Show the biggest subdirectories and files contained within the
-- current directory. Also show the most recent files.
-- The size of a directory includes the sizes of all files
-- and subdirectories contained within it.

-- usage:  ex top 
-- To search your whole disk, first type: cd \
 
without type_check

include file.e
include get.e

constant NDIRS = 50, NFILES = 30, NDATES = 30
constant PAGE_SIZE = 15

constant SCREEN = 1, ERR = 2
constant MEG = 1000000

sequence cmd

sequence dlist, flist, date_list

function top_list(sequence list, object new_item, integer max)
-- add new item to correct place on list, if it's good enough
    integer place
    
    place = length(list)
    while place >= 1 do
	if compare(new_item, list[place]) <= 0 then
	    exit
	end if
	place = place - 1
    end while
    if place < length(list) then
	-- better than some item(s) on the list:
	list = append(list[1..place], new_item) & list[place+1..length(list)] 
	if length(list) > max then
	    list = list[1..max]
	end if
    elsif length(list) < max then
	-- grow the list
	list = append(list, new_item)
    end if
    return list
end function

function xwalk_dir(sequence path_name)
-- walk through a directory and its subdirectories 
-- "looking" at each file
    object d
    atom size, total
    sequence name, full_path, date_time
    
    d = dir(path_name)
    while find(path_name[length(path_name)], " \\") do
	path_name = path_name[1..length(path_name)-1]
    end while
    if atom(d) then
	puts(ERR, "Couldn't open " & path_name & '\n')
	return 0
    end if
    total = 0
    for i = 1 to length(d) do
	name = d[i][D_NAME]
	if not find(name, {".", ".."}) then
	    full_path = path_name & '\\' & name
	    if find('d', d[i][D_ATTRIBUTES]) then
		-- directory
		size = xwalk_dir(full_path)
		dlist = top_list(dlist, {size, full_path}, NDIRS)
	    else
		-- file
		size = d[i][D_SIZE]
		flist = top_list(flist, {size, full_path}, NFILES)
	    
		date_time = d[i][D_YEAR.. D_MINUTE]
		date_list = top_list(date_list, {date_time, full_path}, NDATES)
	    end if
	    total = total + size
	end if
    end for
    return total
end function

function more(integer lines, integer inc)
-- prompt after PAGE_SIZE lines have been printed
    lines = lines - inc
    if lines <= 0 then
	puts(SCREEN, "more...\r")
	if wait_key() then
	end if
	lines = PAGE_SIZE
    end if
    return lines
end function

function trim(sequence path)
-- trim prefix of path name
    if length(path) >= 3 then
	if path[1] = '.' and path[2] = '\\' then
	    return path[3..length(path)]
	elsif path[2] = ':' and path[3] = '\\' then
	    return path[4..length(path)]
	end if
    end if
    return path
end function

cmd = command_line()
if length(cmd) != 2 then
    puts(ERR, "usage: ex space\n")
    abort(1)
end if

atom grand_total

dlist = {}
flist = {}
date_list = {}

sequence start_dir
if sequence(dir(".")) then
    start_dir = "."
else
    start_dir = current_dir()
end if

grand_total = xwalk_dir(start_dir)

integer lines

printf(SCREEN, "The %d biggest directories (Mb)\n\n", length(dlist))
lines = PAGE_SIZE
for i = 1 to length(dlist) do
    printf(SCREEN, "%9.1f  %s\n", {dlist[i][1]/MEG, trim(dlist[i][2])}) 
    lines = more(lines, 1)
end for

printf(SCREEN, "\nThe %d biggest files (Mb)\n\n", length(flist))
lines = more(lines, 3)
for i = 1 to length(flist) do
    printf(SCREEN, "%9.1f  %s\n", {flist[i][1]/MEG, trim(flist[i][2])}) 
    lines = more(lines, 1)
end for

printf(SCREEN, "\nThe %d most recent files\n\n", length(date_list))
lines = more(lines, 3)
for i = 1 to length(date_list) do
    printf(SCREEN, "%4d-%02d-%02d %2d:%02d  %s\n", append(date_list[i][1], 
						     trim(date_list[i][2]))) 
    lines = more(lines, 1)
end for

